import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { F_OK } from 'constants';
import { access, mkdir } from 'fs/promises';
import { join } from 'path';
import { gitP } from 'simple-git';
import { Repository } from 'typeorm';
import { Project } from '../projects/project.entity';
import { ListBranchesArgs } from './dtos/list-branches.args';
import { ListLogsArgs } from './dtos/list-logs.args';
import { ConfigService } from '@nestjs/config';
import { log } from 'console';

@Injectable()
export class ReposService {
  constructor(
    @InjectRepository(Project)
    private readonly projectRepository: Repository<Project>,
    private readonly configService: ConfigService,
  ) {}

  async getGit(project: Project) {
    const workspacePath = join(
      this.configService.get<string>('workspaces.root'),
      project.name,
    );
    const firstInit = await access(workspacePath, F_OK)
      .then(() => false)
      .catch(async () => {
        await mkdir(workspacePath);
        return true;
      });
    const git = gitP(workspacePath);
    if (firstInit) {
      await git.init();
      await git.addRemote('origin', project.sshUrl);
    }
    return git;
  }

  async listLogs(dto: ListLogsArgs) {
    const project = await this.projectRepository.findOneOrFail({
      id: dto.projectId,
    });
    const git = await this.getGit(project);
    await git.fetch();
    return await git.log({
      '--branches': dto.branch ?? '',
      '--remotes': 'origin',
    });
  }

  async listBranches(dto: ListBranchesArgs) {
    const project = await this.projectRepository.findOneOrFail({
      id: dto.projectId,
    });
    const git = await this.getGit(project);
    return git.branch();
  }
}
